<?php

namespace SGalinski\SgNews\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Domain\Model\Category;
use SGalinski\SgNews\Domain\Model\News;
use SGalinski\SgNews\Domain\Repository\CategoryRepository;
use SGalinski\SgNews\Domain\Repository\NewsRepository;
use SGalinski\SgNews\Domain\Repository\TagRepository;
use SGalinski\SgNews\Domain\Service\NewsService;
use SGalinski\SgNews\Service\ConfigurationService;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Controller that shows the latest news
 */
class LatestController extends AbstractController {
	/**
	 * @var CategoryRepository
	 */
	protected $categoryRepository;

	/**
	 * @var TagRepository
	 */
	protected $tagRepository;

	/**
	 * @var NewsRepository
	 */
	protected $newsRepository;

	/**
	 * @var NewsService
	 */
	protected $newsService;

	/**
	 * Renders the news overview
	 *
	 * @param array $newsMetaData
	 * @param int $offset
	 * @return null|\Psr\Http\Message\ResponseInterface
	 * @throws \InvalidArgumentException
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\InvalidQueryException
	 * @throws \TYPO3\CMS\Extbase\Configuration\Exception\InvalidConfigurationTypeException
	 */
	public function indexAction(array $newsMetaData = [], $offset = 0): ?\Psr\Http\Message\ResponseInterface {
		$limit = 0;
		if (isset($this->settings['limit'])) {
			$limit = ((int) $this->settings['limit']);
		}
		$limit = ($limit < 1 ? 1 : $limit);

		$configurationService = GeneralUtility::makeInstance(ConfigurationService::class);
		$sortBy = $configurationService->getConfiguration('sortBy', $this->settings);

		$categoryUids = [];
		$tagUids = [];
		if (isset($this->settings['categories'])) {
			$categoryUids = GeneralUtility::intExplode(',', $this->settings['categories'], TRUE);
		}
		if (isset($this->settings['tags'])) {
			$tagUids = GeneralUtility::intExplode(',', $this->settings['tags'], TRUE);
		}
		if ((int) $GLOBALS['TSFE']->page['doktype'] === 117 && !count($categoryUids)) {
			$categoryUids = [$GLOBALS['TSFE']->id];
		}

		$startTime = 0;
		$endTime = 0;
		if (isset($this->settings['starttime'])) {
			$startTime = (int) $this->settings['starttime'];
		}
		if (isset($this->settings['endtime'])) {
			$endTime = (int) $this->settings['endtime'];
		}
		$latestNewsEntries = $this->newsRepository->findLastUpdatedOrHighlightedNewsByCategories(
			$limit,
			FALSE,
			$categoryUids,
			$offset,
			TRUE,
			$sortBy,
			$tagUids,
			$startTime,
			$endTime
		);

		$categories = [];
		$category = NULL;
		foreach ($latestNewsEntries as $latestNewsEntry) {
			/** @var News $latestNewsEntry */
			/** @var Category $category */
			$categoryUid = $latestNewsEntry->getPid();

			if (isset($categories[$categoryUid])) {
				$category = $categories[$categoryUid];
			}

			if (!$category) {
				$category = $this->categoryRepository->findByUid($categoryUid);
				if (!$category) {
					continue;
				}
				$categories[$categoryUid] = $category;
			}

			$newsMetaData[] = $this->newsService->getMetaDataForNews($latestNewsEntry, $category);
			// Make sure, that the amount is the same as the configured limit.
			if (count($newsMetaData) >= $limit) {
				break;
			}
		}

		$this->view->assign('newsMetaData', $newsMetaData);
		if (version_compare(\TYPO3\CMS\Core\Utility\VersionNumberUtility::getCurrentTypo3Version(), '11.0.0', '<')) {
			return NULL;
		} else {
			return $this->htmlResponse();
		}
	}

	/**
	 * @param CategoryRepository $categoryRepository
	 */
	public function injectCategoryRepository(
		CategoryRepository $categoryRepository
	) {
		$this->categoryRepository = $categoryRepository;
	}

	/**
	 * @param NewsRepository $newsRepository
	 */
	public function injectNewsRepository(NewsRepository $newsRepository) {
		$this->newsRepository = $newsRepository;
	}

	/**
	 * @param TagRepository $tagRepository
	 */
	public function injectTagRepository(TagRepository $tagRepository) {
		$this->tagRepository = $tagRepository;
	}

	/**
	 * @param NewsService $newsService
	 */
	public function injectNewsService(NewsService $newsService) {
		$this->newsService = $newsService;
	}
}
